from rich import console
from rich.console import Console
from rich.traceback import install
from rich.console import Console
from rich.theme import Theme
from rich.prompt import Prompt
from rich.table import Table
from rich.panel import Panel
from rich.color import Color
from rich import pretty, print, inspect
from rich.text import Text
from rich.progress import track
# rich documentation: https://rich.readthedocs.io/en/latest/

class RichText:

    @classmethod
    def dev_test(cls):
        # RichText.table()
        # print(['hello', True])
        # print(Panel.fit('[bold yellow]Hi, I am joker[/]', border_style='red'))
        # color = Color.parse('red')
        # inspect(color)
        # RichText.console.print([1,2,3])
        # RichText.console.print('[blue underline]Look like a link[/]')
        # RichText.console.print(locals())
        # RichText.console.print('foo', style = 'underline link https://www.baidu.com')
        # RichText.console.log('Hello world', log_locals=True)
        # RichText.console.out('Locals', locals())
        # RichText.console.rule('[bold red]Chapter 2', align='center')
        # with RichText.console.status("Workings....", spinner='runner'):
        #     i = 0
        #     while i < 100000000:
        #         i = i + 1

        # RichText.console.input('What is [i]your[/] [bold red]name[/]? :smiley: ')
        # RichText.console.save_html('/Users/joker/Desktop/console.html')
        # RichText.console.print(Panel(Text('Hello', justify='right')))
        # exit(0)
        pass

    theme = Theme({
        'info': 'green bold',
        'warning': 'yellow bold',
        'error': 'red bold',
    })

    console = Console(theme=theme)

    @classmethod
    def better_debug(cls):
        install()
        pretty.install()

    @classmethod
    def warn(cls, text):
        RichText.console.print(text, style='warning')

    @classmethod
    def info(cls, text):
        RichText.console.print(text, style='info')
    
    @classmethod
    def error(cls, text):
        RichText.console.print(text, style='error')

    @classmethod
    def prompt(cls, text, choices=None, default=None):
        return Prompt.ask(text, choices=choices, default=default)

    @classmethod
    def table(cls, title, table_data, caption = None):
        t = Table(
            title = title,
            title_style = 'green bold',
            caption = caption,
            caption_style = 'gray',
            highlight = True,
            show_lines = True,
            show_header = False)
        for row in table_data:
            t.add_row(*row)
        RichText.console.print(t)


# -*- coding: utf8 -*-
import os
import json
import re
from ..utils.utils import makedirs

# https://fabricmc.net/use/

class Fabric:

    _configuration = None

    @classmethod
    def json_configuration(cls, config):
        if Fabric._configuration == None:
            fabric_json_path = os.path.join(config.game_version_client_versions_dir(), config.lastVersionId, config.lastVersionId + '.json')
            if  'fabric' in config.lastVersionId.lower() and os.path.exists(fabric_json_path):
                with open(fabric_json_path, 'r') as f:
                    Fabric._configuration = json.load(f)
        return Fabric._configuration

    @classmethod
    def library_fabric_jar_paths(cls, config):
        ret = []
        for (url, file_path) in Fabric.downloadLibrariesMap(config).items():
            ret.append(file_path)
        return ret

    @classmethod
    def jvmOpts(cls, config):
        ret = []
        for o in Fabric.json_configuration(config).get('arguments').get('jvm'):
            o = re.sub(r'\s','',o)
            ret.append(o)
        return ret

    @classmethod
    def downloadLibrariesMap(cls, config):
        ret = {}
        for l in Fabric.json_configuration(config).get('libraries'):
            name = l.get('name')
            splits = name.split(':')
            splits = splits[0].split('.') + splits[1:]
            file_name = '-'.join(splits[-2:]) + '.jar'
            dir_path = os.path.join(config.game_version_client_library_dir(),*splits)
            makedirs(dir_path)
            url_path ='/'.join(splits + [file_name])
            url = l.get('url') + url_path
            ret[url] = os.path.join(dir_path, file_name)
        return ret
